@@ -27,7 +27,8 @@
 
 #include "buddy.h"
 #include "chat.h"
-#include "google.h"
+#include "google/google.h"
+#include "google/google_roster.h"
 #include "presence.h"
 #include "roster.h"
 #include "iq.h"
@@ -47,12 +48,47 @@ static gchar *roster_groups_join(GSList *list)
 	return g_string_free(out, FALSE);
 }
 
+static void roster_request_cb(JabberStream *js, const char *from,
+                              JabberIqType type, const char *id,
+                              xmlnode *packet, gpointer data)
+{
+	xmlnode *query;
+
+	if (type == JABBER_IQ_ERROR) {
+		/*
+		 * This shouldn't happen in any real circumstances and
+		 * likely constitutes a server-side misconfiguration (i.e.
+		 * explicitly not loading mod_roster...)
+		 */
+		purple_debug_error("jabber", "Error retrieving roster!?\n");
+		jabber_stream_set_state(js, JABBER_STREAM_CONNECTED);
+		return;
+	}
+
+	query = xmlnode_get_child(packet, "query");
+	if (query == NULL) {
+		jabber_stream_set_state(js, JABBER_STREAM_CONNECTED);
+		return;
+	}
+
+	jabber_roster_parse(js, from, type, id, query);
+	jabber_stream_set_state(js, JABBER_STREAM_CONNECTED);
+}
+
 void jabber_roster_request(JabberStream *js)
 {
 	JabberIq *iq;
+	xmlnode *query;
 
 	iq = jabber_iq_new_query(js, JABBER_IQ_GET, "jabber:iq:roster");
+	query = xmlnode_get_child(iq->node, "query");
 
+	if (js->server_caps & JABBER_CAP_GOOGLE_ROSTER) {
+		xmlnode_set_attrib(query, "xmlns:gr", NS_GOOGLE_ROSTER);
+		xmlnode_set_attrib(query, "gr:ext", "2");
+	}
+
+	jabber_iq_set_callback(iq, roster_request_cb, NULL);
 	jabber_iq_send(iq);
 }
 
@@ -155,6 +191,9 @@ void jabber_roster_parse(JabberStream *js, const char *from,
                          JabberIqType type, const char *id, xmlnode *query)
 {
 	xmlnode *item, *group;
+#if 0
+	const char *ver;
+#endif
 
 	if (!jabber_is_own_account(js, from)) {
 		purple_debug_warning("jabber", "Received bogon roster push from %s\n",
@@ -181,18 +220,18 @@ void jabber_roster_parse(JabberStream *js, const char *from,
 			continue;
 
 		if(subscription) {
-			if (jb == js->user_jb)
-				jb->subscription = JABBER_SUB_BOTH;
-			else if(!strcmp(subscription, "none"))
-				jb->subscription = JABBER_SUB_NONE;
-			else if(!strcmp(subscription, "to"))
-				jb->subscription = JABBER_SUB_TO;
-			else if(!strcmp(subscription, "from"))
-				jb->subscription = JABBER_SUB_FROM;
-			else if(!strcmp(subscription, "both"))
-				jb->subscription = JABBER_SUB_BOTH;
-			else if(!strcmp(subscription, "remove"))
+			if (g_str_equal(subscription, "remove"))
 				jb->subscription = JABBER_SUB_REMOVE;
+			else if (jb == js->user_jb)
+				jb->subscription = JABBER_SUB_BOTH;
+			else if (g_str_equal(subscription, "none"))
+				jb->subscription = JABBER_SUB_NONE;
+			else if (g_str_equal(subscription, "to"))
+				jb->subscription = JABBER_SUB_TO;
+			else if (g_str_equal(subscription, "from"))
+				jb->subscription = JABBER_SUB_FROM;
+			else if (g_str_equal(subscription, "both"))
+				jb->subscription = JABBER_SUB_BOTH;
 		}
 
 		if(purple_strequal(ask, "subscribe"))
@@ -218,7 +257,16 @@ void jabber_roster_parse(JabberStream *js, const char *from,
 					seen_empty = TRUE;
 				}
 
-				groups = g_slist_prepend(groups, group_name);
+				/*
+				 * See the note in add_purple_buddy_to_groups; the core handles
+				 * names case-insensitively and this is required to not
+				 * end up with duplicates if a buddy is in, e.g.,
+				 * 'XMPP' and 'xmpp'
+				 */
+				if (g_slist_find_custom(groups, group_name, (GCompareFunc)purple_utf8_strcasecmp))
+					g_free(group_name);
+				else
+					groups = g_slist_prepend(groups, group_name);
 			}
 
 			add_purple_buddy_to_groups(js, jid, name, groups);
@@ -227,13 +275,21 @@ void jabber_roster_parse(JabberStream *js, const char *from,
 		}
 	}
 
-	js->currently_parsing_roster_push = FALSE;
+#if 0
+	ver = xmlnode_get_attrib(query, "ver");
+	if (ver) {
+		 PurpleAccount *account = purple_connection_get_account(js->gc);
+		 purple_account_set_string(account, "roster_ver", ver);
+	}
+#endif
 
-	/* if we're just now parsing the roster for the first time,
-	 * then now would be the time to declare ourselves connected.
-	 */
-	if (js->state != JABBER_STREAM_CONNECTED)
-		jabber_stream_set_state(js, JABBER_STREAM_CONNECTED);
+	if (type == JABBER_IQ_SET) {
+		JabberIq *ack = jabber_iq_new(js, JABBER_IQ_RESULT);
+		jabber_iq_set_id(ack, id);
+		jabber_iq_send(ack);
+	}
+
+	js->currently_parsing_roster_push = FALSE;
 }
 
 /* jabber_roster_update frees the GSList* passed in */
@@ -297,7 +353,7 @@ static void jabber_roster_update(JabberStream *js, const char *name,
 
 	if (js->server_caps & JABBER_CAP_GOOGLE_ROSTER) {
 		jabber_google_roster_outgoing(js, query, item);
-		xmlnode_set_attrib(query, "xmlns:gr", "google:roster");
+		xmlnode_set_attrib(query, "xmlns:gr", NS_GOOGLE_ROSTER);
 		xmlnode_set_attrib(query, "gr:ext", "2");
 	}
 	jabber_iq_send(iq);
