@@ -31,7 +31,7 @@ typedef struct _MsnSlpMessage MsnSlpMessage;
 #include "slpcall.h"
 #include "slplink.h"
 #include "session.h"
-#include "msg.h"
+#include "p2p.h"
 
 #include "slp.h"
 
@@ -45,37 +45,21 @@ struct _MsnSlpMessage
 	MsnSlpLink *slplink; /**< The slplink through which this slp message is being sent. */
 	MsnSession *session;
 
-	long session_id;
-	long id;
-	long ack_id;
-	long ack_sub_id;
-	long long ack_size;
+	MsnP2PInfo *p2p_info;
 
-	gboolean sip; /**< A flag that states if this is a SIP slp message. */
-	long flags;
+	long id;
 
 	gboolean ft;
 	PurpleStoredImage *img;
 	guchar *buffer;
 
-	/**
-	 * For outgoing messages this is the number of bytes from buffer that
-	 * have already been sent out.  For incoming messages this is the
-	 * number of bytes that have been written to buffer.
-	 */
-	long long offset;
-
 	/**
 	 * This is the size of buffer, unless this is an outgoing file transfer,
 	 * in which case this is the size of the file.
 	 */
 	long long size;
 
-	GList *msgs; /**< The real messages. */
-
-#if 1
-	MsnMessage *msg; /**< The temporary real message that will be sent. */
-#endif
+	GList *parts; /**< A list with the SlpMsgParts */
 
 	const char *info;
 	gboolean text_body;
@@ -85,6 +69,7 @@ struct _MsnSlpMessage
  * Creates a new slp message
  *
  * @param slplink The slplink through which this slp message will be sent.
+ * If it's set to NULL, it is a temporary SlpMessage.
  * @return The created slp message.
  */
 MsnSlpMessage *msn_slpmsg_new(MsnSlpLink *slplink);
@@ -96,17 +81,70 @@ MsnSlpMessage *msn_slpmsg_new(MsnSlpLink *slplink);
  */
 void msn_slpmsg_destroy(MsnSlpMessage *slpmsg);
 
+/**
+ * Relate this SlpMessage with an existing SlpLink
+ *
+ * @param slplink 	The SlpLink that will send this message.
+ */
+void msn_slpmsg_set_slplink(MsnSlpMessage *slpmsg, MsnSlpLink *slplink);
+
 void msn_slpmsg_set_body(MsnSlpMessage *slpmsg, const char *body,
 						 long long size);
 void msn_slpmsg_set_image(MsnSlpMessage *slpmsg, PurpleStoredImage *img);
-void msn_slpmsg_open_file(MsnSlpMessage *slpmsg,
-						  const char *file_name);
 MsnSlpMessage * msn_slpmsg_sip_new(MsnSlpCall *slpcall, int cseq,
 								   const char *header,
 								   const char *branch,
 								   const char *content_type,
 								   const char *content);
 
-void msn_slpmsg_show(MsnMessage *msg);
+/**
+ * Create a new SLP Ack message
+ *
+ * @param header the value of the header in this slpmsg.
+ *
+ * @return A new SlpMessage with ACK headers
+ */
+MsnSlpMessage *msn_slpmsg_ack_new(MsnP2PInfo *info);
+
+/**
+ * Create a new SLP message for MsnObject data.
+ *
+ * @param slpcall 	The slpcall that manages this message.
+ * @param img 		The image to be sent in this message.
+ *
+ * @return A new SlpMessage with MsnObject info.
+ */
+MsnSlpMessage *msn_slpmsg_obj_new(MsnSlpCall *slpcall, PurpleStoredImage *img);
+
+/**
+ * Create a new SLP message for data preparation.
+ *
+ * @param slpcall 	The slpcall that manages this message.
+ *
+ * @return A new SlpMessage with data preparation info.
+ */
+MsnSlpMessage *msn_slpmsg_dataprep_new(MsnSlpCall *slpcall);
+
+/**
+ * Create a new SLP message for File transfer.
+ *
+ * @param slpcall 	The slpcall that manages this message.
+ * @param size 		The size of the file being transsmited.
+ *
+ * @return A new SlpMessage with the file transfer info.
+ */
+MsnSlpMessage *msn_slpmsg_file_new(MsnSlpCall *slpcall, size_t size);
+
+/**
+ * Serialize the MsnSlpMessage in a way it can be used to be transmited
+ *
+ * @param slpmsg 	The MsnSlpMessage.
+ * @param ret_size 	The size of the buffer cointaining the message.
+ *
+ * @return a buffer with the serialized data.
+ */
+char *msn_slpmsg_serialize(MsnSlpMessage *slpmsg, size_t *ret_size);
+
+void msn_slpmsg_show_readable(MsnSlpMessage *slpmsg);
 
 #endif /* _MSN_SLPMSG_H_ */
